/*****************************************************************************
 *                                                                           *
 *                Licensed Materials - Property of IBM Corp.                 *
 *                  5648-C60 (c) Copyright IBM Corp. 2000                    *
 *                                                                           *
 *       MQJMS Standalone Application Server Facilities (ASF) Samples        *
 *                                                                           *
 *                                                                           *
 *  FILENAME:     MyServerSession.java                                       *
 *                                                                           *
 *  DESCRIPTION:  A very simple implementation of the JMS ServerSession      *
 *                interface. It is used to associate a thread with a JMS     *
 *                Session. Instances of this class are pooled by a           *
 *                ServerSessionPool.                                         *
 *                                                                           *
 *                Further information about this class and its usage can be  *
 *                found in the 'Websphere MQ Using Java' documentation.      *
 *                                                                           *
 *  USAGE:        n/a                                                        *
 *                                                                           *
 *  HISTORY:      010131 - First released as part of SupportPac MA88 v2      *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;

class MyServerSession implements ServerSession, Runnable {

   private Thread      thread;            // The thread and the JMS Session to
   private Session     session;           //         associate with each other

   private MyServerSessionPool ownerPool;       // The pool this ServerSession
                                                                 // belongs to

   private int         id;            // A simple identity key for this object

   private boolean     ready = false;       // Flags holding the state of this
   private boolean     quit  = false;       //                   ServerSession

   /**************************************************************************
    *  MyServerSession - Constructor                                         *
    **************************************************************************/
   MyServerSession( MyServerSessionPool ownerPool, int id, Session session ) {
      // Save parameters to instance members
      this.ownerPool = ownerPool;
      this.id        = id;
      this.session   = session;

      // Create the thread based on the run method of this class, and start it
      thread = new Thread(this);
      thread.start();
   }

   /**************************************************************************
    *  getSession - Implementation of javax.jms.ServerSession. Simply return *
    *               the JMS Session associated with this ServerSession       *
    **************************************************************************/
   public Session getSession() {
      return session;
   }

   /**************************************************************************
    *  start - Implementation of javax.jms.ServerSession. Called by the ASF  *
    *          internals to kick off this ServerSession.                     *
    **************************************************************************/
   public void start() {
      synchronized(this) {
         ready=true;
         notify();
      }
   }

   /**************************************************************************
    *  close - Triggers events to end the run() method, and closes the JMS   *
    *          session which was associated with this ServerSession          *
    **************************************************************************/
   public void close() {
      synchronized(this) {
         quit=true;
         notify();
      }

      try {
         session.close();
      } catch( JMSException e ) {
         // Some kind of error reporting might be useful here
      }
   }

   /**************************************************************************
    *  run - Implementation of java.lang.Runnable. Waits until we have been  *
    *        notified, then, unless we are in the 'quit' state, call the JMS *
    *        Session's run() method, tidying things up afterwards            *
    **************************************************************************/
   public void run() {
      while(!quit) {
         // All the time we are not in the 'quit' state, exercise this loop

         synchronized(this) {
            while(!ready && !quit) {
               // Loop, waiting to be notified that we have been put into
               // either 'ready' or 'quit' state
               try {
                  wait();
               } catch( InterruptedException ie ) {
                  // Ignore
               }
            }

            // Reset 'ready' for the next time round the outer loop
            ready=false;
         }

         if(!quit) {
            // Provided we have not be placed in the 'quit' state, invoke
            // the JMS Session's run method
            session.run();

            // After the run method completes, notify the pool that owns
            // this ServerSession of this
            ownerPool.serverSessionFinished(id);
         }
      }

      // We must be in the 'quit' state by here, so simply exit this method,
      // and the thread will cease to run
   }
}
